package top.daxian1218.dust.db;

import top.daxian1218.dust.commons.log.LoggerHelper;
import top.daxian1218.dust.commons.thread.LocalHolder;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.sql.*;

import java.util.List;

import static top.daxian1218.dust.commons.generic.Preconditions.checkNotNull;

/**
 * 数据源适配管理器
 *
 * @author hst
 */
public class DbAdapterManager {
    private final static String LOCAL_SQL_ADAPTER_CONTEXT = "Dict:Local:SqlAdapterContext";

    private final DbManager dbManager;

    public DbAdapterManager(DbManager dbManager) {
        checkNotNull(dbManager);
        this.dbManager = dbManager;
    }

    /**
     * 根据数据源名称获取对应的适配类
     *
     * @param name 数据源名称
     * @return 数据库适配器
     */
    public IDataAdapter getAdapter(String name) throws DustDbException {
        return getAdapter(name, false);
    }

    public IDataAdapter getAdapter(String name, boolean abandon) throws DustDbException {
        IDataAdapter adapter = dbManager.newAdapter(name);

        if (!abandon) {
            //压入线程，用于后续释放
            SqlAdapterContext ctx = LocalHolder.getOrDefault(LOCAL_SQL_ADAPTER_CONTEXT, new SqlAdapterContext());
            adapter.setAbandon(false);
            ctx.getAdapterList().add(adapter);
        } else {
            LoggerHelper.warning("创建独立的IDataAdapter, 请注意及时释放");
        }
        return adapter;
    }

    public DbManager getDbManager() {
        return dbManager;
    }

    /**
     * 请在释放ThreadLocal资源前关闭连接
     */
    public static void destroy() {
        SqlAdapterContext ctx = (SqlAdapterContext) LocalHolder.get(LOCAL_SQL_ADAPTER_CONTEXT);
        if (ctx == null) {
            return;
        }

        List<IDataAdapter> adapterList = ctx.getAdapterList();
        for (IDataAdapter adapter : adapterList) {
            adapter.rollbackQuiet();
            adapter.destroy();
        }

        adapterList.clear();
    }
}
