package top.daxian1218.dust.db;

import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.commons.id.IIdWorker;
import top.daxian1218.dust.commons.id.SnowFlakeIdWorker;
import top.daxian1218.dust.commons.thread.LocalHolder;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.cache.ICacheManager;
import top.daxian1218.dust.db.dict.IDictObjContainer;
import top.daxian1218.dust.db.dict.IVariableHandler;
import top.daxian1218.dust.db.dict.SqlBuilder;
import top.daxian1218.dust.db.dict.VariableHandler;
import top.daxian1218.dust.db.support.clickhouse.ClickHouseBuilder;
import top.daxian1218.dust.db.support.highgo.HighgoBuilder;
import top.daxian1218.dust.db.support.mssql.MssqlBuilder;
import top.daxian1218.dust.db.support.mysql.MySqlBuilder;
import top.daxian1218.dust.db.support.oracle.OracleBuilder;
import top.daxian1218.dust.db.support.postgresql.PostgreSqlBuilder;

import java.util.ArrayList;
import java.util.List;

/**
 * 字典全局类
 * @author hst
 */
public class DictGlobalConfig {
    private String dateFormat = "yyyy-MM-dd HH:mm:ss";
    private final static String LOCAL_DATA_ADAPTER = "DictGlobalConfig:DataAdapter";
    private String dictDb;
    private String defaultApp;

    /**
     * 是否调试模式
     * 调试模式下会打印执行SQL
     */
    private boolean debug;
    /**
     * 是否启用字典缓存，默认false
     */
    private boolean noCache;
    private IVariableHandler variableHandler;
    private IDictObjContainer objContainer;
    private ICacheManager cacheManager;

    private IIdWorker idWorker;
    private final List<SqlBuilder> sqlBuilders;

    public DictGlobalConfig() {
        idWorker = new SnowFlakeIdWorker(31,31);
        variableHandler = new VariableHandler();
        sqlBuilders = new ArrayList<>();
        sqlBuilders.add(new MySqlBuilder());
        sqlBuilders.add(new OracleBuilder());
        sqlBuilders.add(new MssqlBuilder());
        sqlBuilders.add(new PostgreSqlBuilder());
        sqlBuilders.add(new ClickHouseBuilder());
        sqlBuilders.add(new HighgoBuilder());
    }

    public static DictGlobalConfig Instance = new DictGlobalConfig();

    public IIdWorker getIdWorker() {
        return idWorker;
    }

    public void setIdWorker(IIdWorker idWorker) {
        this.idWorker = idWorker;
    }

    public ICacheManager getCacheManager() {
        return cacheManager;
    }

    public String getDateFormat() {
        return dateFormat;
    }

    public void setDateFormat(String dateFormat) {
        this.dateFormat = dateFormat;
    }

    public String getDictDb() {
        return dictDb;
    }

    public void setDictDb(String dictDb) {
        this.dictDb = dictDb;
    }

    public String getDefaultApp() {
        return defaultApp;
    }

    public void setDefaultApp(String defaultApp) {
        this.defaultApp = defaultApp;
    }

    public boolean isDebug() {
        return debug;
    }

    public void setDebug(boolean debug) {
        this.debug = debug;
    }

    public boolean isNoCache() {
        return noCache;
    }

    public void setNoCache(boolean noCache) {
        this.noCache = noCache;
    }

    public IDataAdapter getDataAdapter() {
        return LocalHolder.get(LOCAL_DATA_ADAPTER);
    }

    public void setDataAdapter(IDataAdapter dataAdapter) {
        Preconditions.checkNotNull(dataAdapter);
        LocalHolder.set(LOCAL_DATA_ADAPTER, dataAdapter);
    }

    public IVariableHandler getVariableHandler() {
        return variableHandler;
    }

    public void setVariableHandler(IVariableHandler variableHandler) {
        this.variableHandler = variableHandler;
    }

    public IDictObjContainer getObjContainer() {
        return objContainer;
    }

    public void setObjContainer(IDictObjContainer objContainer) {
        this.objContainer = objContainer;
    }

    public List<SqlBuilder> getSqlBuilders() {
        return sqlBuilders;
    }

    public void setCacheManager(ICacheManager cacheManager) {
        this.cacheManager = cacheManager;
    }
}
