package top.daxian1218.dust.db;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.exception.DustThirdException;
import top.daxian1218.dust.commons.log.LoggerHelper;
import top.daxian1218.dust.db.base.HDataCommand;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.db.util.DbUtil;

import java.sql.SQLException;
import java.util.Optional;

/**
 * 数据库异常，不包括有关Sql的异常
 *
 * @author hst
 */
public class DustDbException extends DustThirdException {
    private String sql;
    private Iterable<Object> parameters;

    public DustDbException(String message, String error, Exception inner) {
        super(message, Optional.ofNullable(error).orElse("0001"), inner);
    }

    public DustDbException(String message, String error) {
        this(message, error, null);
    }

    public DustDbException(String message) {
        this(message, null, null);
    }

    public void setSql(String sql) {
        this.sql = sql;
    }

    public String getSql() {
        return sql;
    }

    public Iterable<Object> getParameters() {
        return parameters;
    }

    public void setParameters(Iterable<Object> parameters) {
        this.parameters = parameters;
    }

    public static DustDbException fromDbException(SQLException ex) {
        return fromDbException(ex, null, null);
    }

    public static DustDbException fromDbException(SQLException ex, HDataCommand cmd) {
        return fromDbException(ex, cmd, null);
    }

    public static DustDbException fromDbException(SQLException ex, HDataCommand cmd, String message) {
        if (message == null) {
            message = "数据库发生异常";
        }

        DustDbException dex = new DustDbException(message, "0004", ex);
        if (cmd != null) {
            dex.setSql(cmd.getExecuteText());
            dex.setParameters(cmd.getParameters().values());
            if (cmd instanceof HSqlCommand) {
                String msg = ex.getMessage();
                if (ex.getCause() != null && !StringUtils.isEmpty(ex.getCause().getMessage())) {
                    msg = msg + ":" + ex.getCause().getMessage();
                }
                DbUtil.writeLog(ex.getMessage(), (HSqlCommand) cmd);
            }

            LoggerHelper.error("[SQL]" + dex.getSql());
        }
        return dex;
    }

    public static DustDbException fromDuplicationException(SQLException ex) {
        return fromDuplicationException(ex, null);
    }

    public static DustDbException fromDuplicationException(SQLException ex, HDataCommand cmd) {
        DustDbException dex = new DustDbException("数据已存在", "0002", ex);
        if (cmd != null) {
            dex.setSql(cmd.getCommandText());
            dex.setParameters(cmd.getParameters().values());
        }
        return dex;
    }

    public static DustDbException fromMissingException(SQLException ex) {
        return fromMissingException(ex, null);
    }

    public static DustDbException fromMissingException(SQLException ex, HDataCommand cmd) {
        DustDbException dex = new DustDbException("数据已删除", "0005", ex);
        if (cmd != null) {
            dex.setSql(cmd.getCommandText());
            dex.setParameters(cmd.getParameters().values());
        }
        return dex;
    }

}
