package top.daxian1218.dust.db.base;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.util.ClassUtil;

import java.util.Objects;

/**
 * 分布式环境下的数据库接入口
 * @see #cluster
 * 对应于数据库实例
 * @see #name
 * 对应于连接池的名字
 * @see #permission
 * 预留字段，后续纳入租户的数据控制管理
 * @author hst
 */
public class DbAccess {
    private String id;

    private String name;
    private DbCluster cluster = DbCluster.ReadWrite;
    private String host;
    private String user;
    private String password;
    private String permission;
    private String validationQuery;
    private String dbType;
    private int status = 1;
    private String queryString;
    private String dbOperationsClass;
    private String dbName;

    public String getHost() {
        return host;
    }

    public void setHost(String host) {
        this.host = host;
    }

    public String getUser() {
        return user;
    }

    public void setUser(String user) {
        this.user = user;
    }

    public String getPassword() {
        return password;
    }

    public void setPassword(String password) {
        this.password = password;
    }

    public String getPermission() {
        return permission;
    }

    public void setPermission(String permission) {
        this.permission = permission;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public DbCluster getCluster() {
        return cluster;
    }

    public void setCluster(DbCluster cluster) {
        this.cluster = cluster;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getValidationQuery() {
        return validationQuery;
    }

    public void setValidationQuery(String validationQuery) {
        this.validationQuery = validationQuery;
    }

    public String getDbType() {
        return dbType;
    }

    public void setDbType(String dbType) {
        this.dbType = dbType;
    }

    public int getStatus() {
        return status;
    }

    public void setStatus(int status) {
        this.status = status;
    }

    public String getQueryString() {
        return queryString;
    }

    public void setQueryString(String queryString) {
        this.queryString = queryString;
    }

    public String getDbOperationsClass() {
        return dbOperationsClass;
    }

    public void setDbOperationsClass(String dbOperationsClass) {
        this.dbOperationsClass = dbOperationsClass;
    }

    public String getDbName() {
        return dbName;
    }

    public void setDbName(String dbName) {
        this.dbName = dbName;
    }

    @Override
    public DbAccess clone() {
        return (DbAccess) ClassUtil.copyClass(this);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        DbAccess dbAccess = (DbAccess) o;
        return status == dbAccess.status &&
                StringUtils.equals(name, dbAccess.name) &&
                StringUtils.equals(host, dbAccess.host) &&
                StringUtils.equals(user, dbAccess.user) &&
                StringUtils.equals(password, dbAccess.password) &&
                StringUtils.equals(validationQuery, dbAccess.validationQuery) &&
                StringUtils.equals(dbType, dbAccess.dbType) &&
                StringUtils.equals(queryString, dbAccess.queryString) &&
                StringUtils.equals(dbOperationsClass, dbAccess.dbOperationsClass) &&
                StringUtils.equals(dbName, dbAccess.dbName);
    }

    @Override
    public int hashCode() {
        return Objects.hash(name, host, user, password, validationQuery, dbType, status, queryString, dbOperationsClass, dbName);
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        sb.append("\n\tname:\"");
        sb.append(this.getName());
        sb.append("\"");

        sb.append("\n\thost:\"");
        sb.append(this.getHost());
        sb.append("\"");

        sb.append("\n\tdbName:\"");
        sb.append(this.getDbName());
        sb.append("\"");

        sb.append("\n\tcluster:\"");
        sb.append(this.getCluster());
        sb.append("\"");

        sb.append("\n\t}");
        return sb.toString();
    }
}
