package top.daxian1218.dust.db.cache;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.generic.Preconditions;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

import static top.daxian1218.dust.db.DbConstants.*;

/**
 * @author hst
 */
public class CacheManager implements ICacheManager {
    public CacheManager() {
        this.register(CACHE_CAFFEINE, new CaffeineCache(5, 10_000));
        this.register(CACHE_CAFFEINE_LONG, new CaffeineCache(30, 10_000));
        this.register(CACHE_MEMORY, new SimpleMemoryCache());
    }

    private static final ConcurrentHashMap<String, ICache> CACHES = new ConcurrentHashMap<>();

    @Override
    public ICache getCache(String name) {
        return CACHES.getOrDefault(name, null);
    }

    @Override
    public List<String> getCacheNames() {
        return Collections.list(CACHES.keys());
    }

    @Override
    public void register(String name, ICache cache) {
        Preconditions.checkNotNull(cache);
        Preconditions.checkNotEmpty(name);

        CACHES.put(name, cache);
    }

    @Override
    public void flush(String name) {
        if (StringUtils.isEmpty(name)) {
            CACHES.forEach((k, v) -> {
                v.clear();
            });

            return;
        }

        if(CACHES.containsKey(name)) {
            CACHES.get(name).clear();
        }

    }
}
