package top.daxian1218.dust.db.cache;

import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author daxian
 */
public class CaffeineCache implements ICache {
    private final Cache<String, Object> caffeineCache;

    public CaffeineCache(long expire, long size) {
        caffeineCache = Caffeine.newBuilder()
             .expireAfterWrite(expire, TimeUnit.MINUTES)
            .maximumSize(size)
            .build();
    }

    @Override
    public void put(String key, Object value) {
        caffeineCache.put(key, value);
    }

    @Override
    public void put(Map<String, Object> keyValues) {
        caffeineCache.putAll(keyValues);
    }

    @Override
    public Object get(String key) {
        return caffeineCache.getIfPresent(key);
    }

    @Override
    public boolean exists(String key) {
        return caffeineCache.asMap().containsKey(key);
    }

    @Override
    public void clear() {
        caffeineCache.invalidateAll();
    }

    @Override
    public List<String> keys() {
        return new ArrayList<>(caffeineCache.asMap().keySet());
    }

    @Override
    public void evict(String... keys) {
        caffeineCache.invalidateAll(Arrays.asList(keys));
    }
}
