package top.daxian1218.dust.db.dict;

import com.alibaba.fastjson.JSONObject;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.DbConstants;
import top.daxian1218.dust.db.DictGlobalConfig;
import top.daxian1218.dust.db.cache.ICache;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.db.cache.SimpleMemoryCache;

import static top.daxian1218.dust.commons.generic.Preconditions.checkNotNull;

/**
 * convert class for dict schema
 *
 * @author hst
 */
public class DictCache {
    private final static String PREFIX_LOCAL_SCHEMA = "DICT:DICT_OBJ";
    private final static DictCache instance = new DictCache();

    public static DictCache getInstance() {
        return instance;
    }

    private ICache memoryCache;

    private ICache getCache() {
        if (DictGlobalConfig.Instance.getCacheManager() != null) {
            ICache dictCache = DictGlobalConfig.Instance.getCacheManager().getCache(DbConstants.CACHE_DICT);
            if (dictCache != null) {
                return dictCache;
            }

            this.memoryCache = new SimpleMemoryCache();
            DictGlobalConfig.Instance.getCacheManager().register(DbConstants.CACHE_DICT, this.memoryCache);
            return this.memoryCache;
        }

        if (memoryCache != null) {
            return memoryCache;
        }

        this.memoryCache = new SimpleMemoryCache();
        return this.memoryCache;
    }

    public String getCacheValue(String key) {
        if (DictGlobalConfig.Instance.isNoCache()) {
            return null;
        }

        return (String) getCache().get(key);
    }

    public void setCacheValue(String key, Object value) {
        if (DictGlobalConfig.Instance.isNoCache()) {
            return ;
        }
        getCache().put(key, TypeUtil.toStringOrJson(value));
    }

    private String genIdxCacheKey(String app, String code) {
        return String.format("%s:APP|CODE:%s|%s", PREFIX_LOCAL_SCHEMA, app, code);
    }

    private String genObjCacheKey(long id) {
        return String.format("%s:ID:%s", PREFIX_LOCAL_SCHEMA, id);
    }

    public long getObjIdFromCache(String app, String code) {
        if (DictGlobalConfig.Instance.isNoCache()) {
            return 0L;
        }
        String cacheKey = genIdxCacheKey(app, code);
        return TypeUtil.toLong(getCacheValue(cacheKey));
    }

    public void saveObjIdToCache(long id, String app, String code) {
        String cacheKey = genIdxCacheKey(app, code);
        setCacheValue(cacheKey, id);
    }

    public JSONObject getCacheSchemaJson(long id) {
        if (id <= 0L) {
            return null;
        }

        String value = getCacheValue(genObjCacheKey(id));
        if (StringUtils.isEmpty(value)) {
            return null;
        }

        return JSONObject.parseObject(value);
    }

    public void saveObjSchemeToCache(JSONObject json) {
        Long id = json.getLong("id");
        String code = json.getString("code");
        String appCode = json.getString("appCode");

        String cacheKey = genObjCacheKey(id);
        setCacheValue(cacheKey, json.toString());
        updateSchemaIndexOfCache(appCode, code, id);
    }

    private void updateSchemaIndexOfCache(String app, String code, long id) {
        String cacheKey = genIdxCacheKey(app, code);
        setCacheValue(cacheKey, id);
    }
}
