package top.daxian1218.dust.db.dict;

import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.db.DictGlobalConfig;

import java.util.ArrayList;
import java.util.List;

/**
 * @author hst
 */
public class DictObjBuilder {
    private String appCode;
    private String objCode;
    private final List<DictObjColumn> columns = new ArrayList<>();
    private final List<DictObjTable> tables = new ArrayList<>();
    private String tableName;
    private String name;

    public DictObjBuilder appCode(String value) {
        appCode = value;
        return this;
    }

    public DictObjBuilder objCode(String value) {
        objCode = value;
        return this;
    }

    public DictObjBuilder tableName(String value) {
        tableName = value;
        return this;
    }

    public DictObjBuilder name(String value) {
        name = value;
        return this;
    }

    public DictObjBuilder addColumn(DictObjColumn col) {
        columns.add(col);
        return this;
    }

    public DictObjBuilder bindTable(DictObjTable table) {
        tables.add(table);
        return this;
    }

    private DictObjTable cloneTable(DictObjTable table) {
        return DictObjTable.parse(table.toJson());
    }

    public DictObj build() {
        DictObj obj = new DictObj();
        obj.setAppCode(appCode);
        obj.setName(name);
        obj.setCode(objCode);
        obj.setTableName(tableName);

        columns.forEach(col -> {
            obj.getColumns().add(col.clone());
        });

        tables.forEach(t -> {
            obj.getRelationTables().add(cloneTable(t));
        });

        return obj;
    }

    public static DictObj create(String code, String appCode) {
        Preconditions.checkNotEmpty(code, "字典编码为空");
        Preconditions.checkNotNull(DictGlobalConfig.Instance.getObjContainer(), "字典对象容器未找到");

        DictObj obj = new DictObj();
        obj.setCode(code);
        obj.setAppCode(appCode);

        return DictGlobalConfig.Instance.getObjContainer().load(obj);
    }

    public static DictObj create(long id) {
        Preconditions.checkNotNull(DictGlobalConfig.Instance.getObjContainer(), "字典对象容器未找到");

        DictObj obj = new DictObj();
        obj.setId(id);

        return DictGlobalConfig.Instance.getObjContainer().load(obj);
    }
}
