package top.daxian1218.dust.db.dict;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.annotation.JSONField;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.data.StatType;
import top.daxian1218.dust.commons.util.ClassUtil;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.dict.ref.RefInfo;
import top.daxian1218.dust.db.dict.ref.RefType;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 字典对象列
 *
 * @author hst
 */
public class DictObjColumn {
    public static final String COLUMN_SUFFIX = "$";

    public DictObjColumn(String columnName) {
        this.columnName = columnName;
    }

    @JSONField(deserialize = false, serialize = false)
    private DictObj dictObj;

    private long id;
    private String columnLabel;
    private String columnName;
    private String physicalName;
    private String tableName;
    private boolean primaryKey;
    private boolean ignore;
    private String mirrorColumn;
    private String relationTable;
    private String relationColumn;
    private boolean required;
    private int dataWidth;
    private int dataDec;
    private DataType dataType = DataType.STRING;
    private StatType statType = StatType.NONE;

    private BigDecimal statValue;

    private String defaultValue;

    private String paraValue;
    private boolean autoIncrement;
    private boolean keyword;
    private RefInfo refInfo;
    private final DictPropertyCollection extendedProperties = new DictPropertyCollection();

    public DictObj getDictObj() {
        return dictObj;
    }

    protected void setDictObj(DictObj dictObj) {
        this.dictObj = dictObj;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getColumnLabel() {
        return columnLabel;
    }

    public void setColumnLabel(String columnLabel) {
        this.columnLabel = columnLabel;
    }

    public String getColumnName() {
        return columnName;
    }

    public String getRefColumnName() {
        return columnName + COLUMN_SUFFIX;
    }

    private void setColumnName(String columnName) {
        this.columnName = columnName;
    }

    public String getPhysicalName() {
        return physicalName;
    }

    public void setPhysicalName(String physicalName) {
        this.physicalName = physicalName;
    }

    public String getTableName() {
        return tableName;
    }

    public void setTableName(String tableName) {
        this.tableName = tableName;
    }

    public boolean isPrimaryKey() {
        return primaryKey;
    }

    public void setPrimaryKey(boolean primaryKey) {
        this.primaryKey = primaryKey;
    }

    public boolean isIgnore() {
        return ignore;
    }

    public void setIgnore(boolean ignore) {
        this.ignore = ignore;
    }

    public String getRefNameColumn() {
        return this.columnName + DictConstant.REF_COL_SUF;
    }

    public String getMirrorColumn() {
        return mirrorColumn;
    }

    public void setMirrorColumn(String mirrorColumn) {
        this.mirrorColumn = mirrorColumn;
    }

    public boolean isRequired() {
        return required;
    }

    public void setRequired(boolean required) {
        this.required = required;
    }

    public int getDataWidth() {
        return dataWidth;
    }

    public void setDataWidth(int dataWidth) {
        this.dataWidth = dataWidth;
    }

    public int getDataDec() {
        return dataDec;
    }

    public void setDataDec(int dataDec) {
        this.dataDec = dataDec;
    }

    public DataType getDataType() {
        return dataType;
    }

    public void setDataType(DataType dataType) {
        this.dataType = dataType;
    }

    public String getDefaultValue() {
        return defaultValue;
    }

    public void setDefaultValue(String defaultValue) {
        this.defaultValue = defaultValue;
    }

    public boolean isAutoIncrement() {
        return autoIncrement;
    }

    public void setAutoIncrement(boolean autoIncrement) {
        this.autoIncrement = autoIncrement;
    }

    public boolean isKeyword() {
        return keyword;
    }

    public void setKeyword(boolean keyword) {
        this.keyword = keyword;
    }

    public boolean isVarValue() {
        return defaultValue != null && defaultValue.startsWith("@");
    }

    public String getRelationTable() {
        return relationTable;
    }

    public void setRelationTable(String relationTable) {
        this.relationTable = relationTable;
    }

    public String getRelationColumn() {
        return relationColumn;
    }

    public void setRelationColumn(String relationColumn) {
        this.relationColumn = relationColumn;
    }

    public RefInfo getRefInfo() {
        return refInfo;
    }

    public void setRefInfo(RefInfo refInfo) {
        this.refInfo = refInfo;
    }

    public DictPropertyCollection getExtendedProperties() {
        return extendedProperties;
    }

    @Override
    public DictObjColumn clone() {
        DictObjColumn dataColumn = new DictObjColumn(this.columnName);
        dataColumn.physicalName = this.physicalName;
        dataColumn.columnLabel = this.columnLabel;
        dataColumn.tableName = this.tableName;
        dataColumn.ignore = this.ignore;
        dataColumn.primaryKey = this.primaryKey;
        dataColumn.required = this.required;
        dataColumn.mirrorColumn = this.mirrorColumn;
        dataColumn.defaultValue = this.defaultValue;
        dataColumn.dataWidth = this.dataWidth;
        dataColumn.dataDec = this.dataDec;
        dataColumn.relationColumn = this.relationColumn;
        dataColumn.relationTable = this.relationTable;
        if (refInfo != null) {
            dataColumn.refInfo = (RefInfo) ClassUtil.copyClass(refInfo);
        }
        return dataColumn;
    }

    public Object toValue(Object value) {
        switch (dataType) {
            case NUMBER:
            case CURRENCY:
                return TypeUtil.toDecimal(value);
            case INT:
                return TypeUtil.toInteger(value);
            case UBIGINT:
                return TypeUtil.toLong(value);
            case STRING:
            case TEXT:
                return TypeUtil.toString(value);
            case BINARY:
                return TypeUtil.toBytes(value);
            case DATE:
                if (value instanceof String) {
                    String str = (String) value;
                    if (str.startsWith("@")) {
                        Object v = dictObj.getVariableHandler().getVariable(str);
                        if (v instanceof Date) {
                            return v;
                        }

                        return TypeUtil.toDateTime(v);
                    }

                }

                if (value instanceof Date) {
                    return value;
                }

                return TypeUtil.toDateTime(value);
            case BOOLEAN:
                return TypeUtil.toBoolean(value);
            default:
                throw new DustDictException("列类型无法转化值");
        }
    }

    public JSONObject toJson() {
        JSONObject schemaJson = new JSONObject();
        schemaJson.put("id", this.id);
        schemaJson.put("physicalName", this.physicalName);
        schemaJson.put("columnLabel", this.columnLabel);
        schemaJson.put("columnName", this.columnName);
        schemaJson.put("tableName", this.tableName);
        schemaJson.put("isPrimaryKey", this.primaryKey);
        schemaJson.put("isAutoIncrement", this.autoIncrement);
        schemaJson.put("isKeyword", this.keyword);
        schemaJson.put("isIgnore", this.ignore);
        schemaJson.put("isRequired", this.required);
        schemaJson.put("mirrorColumn", this.mirrorColumn);
        schemaJson.put("dataType", this.dataType);
        schemaJson.put("defaultValue", this.defaultValue);
        schemaJson.put("dataWidth", this.dataWidth);
        schemaJson.put("dataDec", this.dataDec);
        schemaJson.put("relationColumn", this.relationColumn);
        schemaJson.put("relationTable", this.relationTable);

        if (this.refInfo != null) {
            schemaJson.put("refInfo", JSON.toJSON(refInfo));
        }

        return schemaJson;
    }

    public static DictObjColumn parse(JSONObject schema) {

        String strDataType = TypeUtil.toString(schema.get("dataType"));
        if (strDataType == null) {
            throw new IllegalArgumentException("dataType is not compatible");
        }

        DataType dataType;
        try {
            dataType = DataType.valueOf(strDataType);
        } catch (Exception e) {
            throw new IllegalArgumentException("dataType is not compatible");
        }

        DictObjColumn dictColumn = new DictObjColumn(TypeUtil.toString(schema.get("columnName")));
        dictColumn.setDataType(dataType);

        if (schema.containsKey("physicalName")) {
            dictColumn.physicalName = TypeUtil.toString(schema.get("physicalName"));
        }

        if (schema.containsKey("columnName")) {
            dictColumn.columnName = TypeUtil.toString(schema.get("columnName"));
        }

        if (schema.containsKey("columnLabel")) {
            dictColumn.columnLabel = TypeUtil.toString(schema.get("columnLabel"));
        }

        if (schema.containsKey("tableName")) {
            dictColumn.tableName = TypeUtil.toString(schema.get("tableName"));
        }

        if (schema.containsKey("ignore")) {
            dictColumn.ignore = TypeUtil.toBoolean(schema.get("ignore"));
        } else if (schema.containsKey("isIgnore")) {
            dictColumn.ignore = TypeUtil.toBoolean(schema.get("isIgnore"));
        }

        if (schema.containsKey("primaryKey")) {
            dictColumn.primaryKey = TypeUtil.toBoolean(schema.get("primaryKey"));
        } else if (schema.containsKey("isPrimaryKey")) {
            dictColumn.primaryKey = TypeUtil.toBoolean(schema.get("isPrimaryKey"));
        }


        if (schema.containsKey("autoIncrement")) {
            dictColumn.autoIncrement = TypeUtil.toBoolean(schema.get("autoIncrement"));
        } else if (schema.containsKey("isAutoIncrement")) {
            dictColumn.autoIncrement = TypeUtil.toBoolean(schema.get("isAutoIncrement"));
        }

        if (schema.containsKey("keyword")) {
            dictColumn.keyword = TypeUtil.toBoolean(schema.get("keyword"));
        } else if (schema.containsKey("isKeyword")) {
            dictColumn.keyword = TypeUtil.toBoolean(schema.get("isKeyword"));
        }

        if (schema.containsKey("required")) {
            dictColumn.required = TypeUtil.toBoolean(schema.get("required"));
        } else if (schema.containsKey("isRequired")) {
            dictColumn.required = TypeUtil.toBoolean(schema.get("isRequired"));
        }


        if (schema.containsKey("mirrorColumn")) {
            dictColumn.mirrorColumn = TypeUtil.toString(schema.get("mirrorColumn"));
        }

        if (schema.containsKey("defaultValue")) {
            dictColumn.defaultValue = TypeUtil.toString(schema.get("defaultValue"));
        }

        if (schema.containsKey("dataWidth")) {
            dictColumn.dataWidth = TypeUtil.toInteger(schema.get("dataWidth"));
        }

        if (schema.containsKey("dataDec")) {
            dictColumn.dataDec = TypeUtil.toInteger(schema.get("dataDec"));
        }

        if (schema.containsKey("relationColumn")) {
            dictColumn.relationColumn = TypeUtil.toString(schema.get("relationColumn"));
        }

        if (schema.containsKey("relationTable")) {
            dictColumn.relationTable = TypeUtil.toString(schema.get("relationTable"));
        }

        if (schema.containsKey("refInfo")) {
            dictColumn.refInfo = JSON.toJavaObject(schema.getJSONObject("refInfo"), RefInfo.class);
        }

        String refType = schema.getString("refType");
        if (!StringUtils.isEmpty(refType) && !StringUtils.equals("NONE", refType)) {
            dictColumn.refInfo = new RefInfo();
            dictColumn.refInfo.setRefType(RefType.valueOf(refType));
            dictColumn.refInfo.setRefObj(schema.getString("refObj"));
            dictColumn.refInfo.setRefColumn(schema.getString("refColumn"));
            dictColumn.refInfo.setRefColumnName(schema.getString("refColumnName"));
            dictColumn.refInfo.setRefQueryWhere(schema.getString("refQueryWhere"));
            dictColumn.refInfo.setRefEditWhere(schema.getString("refEditWhere"));
            dictColumn.refInfo.setStrictRef(schema.getBooleanValue("isStrictRef"));
        }

        if (StringUtils.isEmpty(dictColumn.physicalName)) {
            dictColumn.ignore = true;
        }


        String strExProps = schema.getString("extendedProperties");
        if (!StringUtils.isEmpty(strExProps) && strExProps.startsWith("{")) {
            JSONObject exProps = JSONObject.parseObject(strExProps);
            for (String k : exProps.keySet()) {
                dictColumn.extendedProperties.add(k, exProps.getString(k));
            }
        }


        return dictColumn;
    }

    @Override
    public String toString() {
        return columnName + "-" + columnLabel;
    }

    public StatType getStatType() {
        return statType;
    }

    public void setStatType(StatType statType) {
        this.statType = statType;
    }

    public BigDecimal getStatValue() {
        return statValue;
    }

    public void setStatValue(BigDecimal statValue) {
        this.statValue = statValue;
    }

    public String getParaValue() {
        return paraValue;
    }

    public void setParaValue(String paraValue) {
        this.paraValue = paraValue;
    }
}
