package top.daxian1218.dust.db.dict;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.generic.IDisposable;
import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.commons.util.MapUtil;
import top.daxian1218.dust.db.dict.event.ActionType;

import java.util.*;

/**
 * @author hst
 */
public class DictObjColumnCollection implements Iterable<DictObjColumn>, IDisposable {
    private final List<DictObjColumn> data = new ArrayList<>();
    private final Map<String, DictObjColumn> index = new HashMap<>();

    private final Map<String, List<String>> relationColumnMap = new HashMap<>();
    private DictObj dictObj;

    public DictObjColumnCollection(DictObj obj) {
        Preconditions.checkNotNull(obj);

        dictObj = obj;
    }

    @Override
    public Iterator<DictObjColumn> iterator() {
        return data.iterator();
    }

    public DictObj getDictObj() {
        return dictObj;
    }

    public void setDictObj(DictObj dictObj) {
        this.dictObj = dictObj;
    }

    public void add(DictObjColumn dc) {
        Preconditions.checkNotNull(dc);

        if (dc.getDictObj() != null) {
            throw new DustDictException("对象列已添加过，无法再添加");
        }

        if (index.containsKey(dc.getColumnName().toUpperCase())) {
            throw new DustDictException("列" + dc.getColumnName() + "已存在");
        }

        dc.setDictObj(dictObj);
        updateIndex(dc, false);
        data.add(dc);
        if (!StringUtils.isEmpty(dc.getMirrorColumn())) {
            MapUtil.putListValue(relationColumnMap, dc.getMirrorColumn().toUpperCase(), dc.getColumnName());
        }
        dictObj.onColumnsChanged(ActionType.ADD, dc);
    }

    public List<String> getRelationColumns(String columnName) {
        return relationColumnMap.get(columnName);
    }

    public boolean contains(DictObjColumn dc) {
        return data.contains(dc);
    }

    public void clear() {
        data.clear();
        index.clear();
        dictObj.onColumnsChanged(ActionType.RESET, null);
    }

    public boolean remove(DictObjColumn dc) {
        Preconditions.checkNotNull(dc);
        if (!data.contains(dc)) {
            return false;
        }

        dc.setDictObj(null);
        updateIndex(dc, true);
        data.remove(dc);

        dictObj.onColumnsChanged(ActionType.REMOVE, dc);
        return true;
    }

    public int size() {
        return data.size();
    }

    private void updateIndex(DictObjColumn col, boolean remove) {
        String upper = col.getColumnName().toUpperCase();
        if (remove) {
            index.remove(upper);
        } else {
            index.put(upper, col);
        }
    }

    public DictObjColumn get(int index) {
        return index < data.size() && index >= 0 ? data.get(index) : null;
    }

    public DictObjColumn get(String name) {
        Preconditions.checkNotEmpty(name);
        String key = name.toUpperCase();
        if (index.containsKey(key)) {
            return index.get(key);
        }

        return null;
    }

    public boolean contains(String name) {
        if (StringUtils.isEmpty(name)) { return false; }
        
        String key = name.toUpperCase();
        return index.containsKey(key);
    }

    public List<DictObjColumn> toList() {
        return new ArrayList<>(data);
    }

    @Override
    public void dispose() {
        data.clear();
        index.clear();
        dictObj = null;
    }
}
