package top.daxian1218.dust.db.dict;

import com.alibaba.fastjson.annotation.JSONField;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.generic.IDisposable;
import top.daxian1218.dust.commons.util.TypeUtil;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static top.daxian1218.dust.commons.generic.Preconditions.checkNotNull;

/**
 * 数据单元，一行一列确定一个数据集的位置，位置对应一个数据单元
 * @deprecated Field会导致内存上的增加以及浪费，已不推荐使用。值操作直接使用DictObjRow上的方法
 * @see DictObjRow
 * @see DictObjRow#setValue(String, Object) 
 * @see DictObjRow#getValue(String)
 * @author hst on 2020-8-1.
 */
@Deprecated
public class DictObjField {
    @JSONField(serialize = false, deserialize = false)
    private final DictObjRow row;
    @JSONField(serialize = false, deserialize = false)
    private final DictObjColumn column;

    public DictObjField(DictObjRow row, DictObjColumn col) {
        checkNotNull(row);
        checkNotNull(col);

        if (row.getDictObj() != col.getDictObj()) {
            throw new DustDictException("行与列不匹配");
        }

        this.row = row;
        this.column = col;
    }

    public DictObj getDictObj() {
        return column.getDictObj();
    }

    public Object getOriginValue() {
        return row.getValue(column, true);
    }

    public Object getValue() {
        return this.row.getValue(this.column);
    }

    public String getRefValue() {
        return TypeUtil.toString(row.getValue(column.getRefNameColumn()));
    }

    public void setValue(Object v, String refValue) {
        this.row.setValue(column, v, refValue);
    }

    public void setValue(Object v) {
        this.setValue(v, null);
    }

    public DictObjRow getRow() {
        return row;
    }

    public DictObjColumn getColumn() {
        return column;
    }

    @Override
    public String toString() {
        return this.column.getColumnLabel() + ":" + this.row.getValue(this.column);
    }


    public String getText() {
        return this.row.getText(column.getColumnName());
    }

    public String getStringValue() {
        return TypeUtil.toString(getValue());
    }

    public boolean getBooleanValue() {
        return TypeUtil.toBoolean(getValue());
    }

    public int getInt32Value() {
        return TypeUtil.toInteger(getValue());
    }

    public long getLongValue() {
        return TypeUtil.toLong(getValue());
    }

    public Date getDateTimeValue() {
        return TypeUtil.toDateTime(getValue());
    }

    public double getDoubleValue() {
        return TypeUtil.toDouble(getValue());
    }

    public BigDecimal getDecimalValue() {
        return TypeUtil.toDecimal(getValue());
    }
}
