package top.daxian1218.dust.db.dict;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.db.dict.condition.DictCondition;

import java.util.ArrayList;
import java.util.List;

import static top.daxian1218.dust.commons.generic.Preconditions.checkNotNull;


/**
 * @author hst on 2020-8-1.
 */
public class DictObjTable {
    private long id;
    private String tableName;
    private String tableLabel;
    private String tableAlias;
    private RelationType relationType = RelationType.LEFT;
    private String relationTable;
    private String relationColumn;
    private String thisColumn;
    private String relationWhere;
    private boolean followDelete = false;
    private boolean followInsert = false;
    private boolean followUpdate = true;
    private boolean primary;
    private final List<DictCondition> conditions = new ArrayList<>();

    public DictObjTable() {
    }

    public DictObjTable(String tableName) {
        this.tableName = tableName;
    }

    public DictObjTable(String tableName, String tableAlias)
    {
        this.tableName = tableName;
        this.tableAlias = tableAlias;
    }

    public String getTableName() {
        return tableName;
    }

    public void setTableName(String tableName) {
        this.tableName = tableName;
    }

    public String getTableLabel() {
        if (StringUtils.isEmpty(tableLabel)) {
            return tableName;
        }
        return tableLabel;
    }

    public void setTableLabel(String tableLabel) {
        this.tableLabel = tableLabel;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public RelationType getRelationType() {
        return relationType;
    }

    public void setRelationType(RelationType relationType) {
        this.relationType = relationType;
    }

    public String getTableAlias() {
        return tableAlias;
    }

    public void setTableAlias(String tableAlias) {
        this.tableAlias = tableAlias;
    }

    public String getRelationTable() {
        return relationTable;
    }

    public void setRelationTable(String relationTable) {
        this.relationTable = relationTable;
    }

    public boolean isPrimary() {
        return primary;
    }

    public void setPrimary(boolean primary) {
        this.primary = primary;
    }

    public List<DictCondition> getConditions() {
        return conditions;
    }

    public String getRelationColumn() {
        return relationColumn;
    }

    public void setRelationColumn(String relationColumn) {
        this.relationColumn = relationColumn;
    }

    public String getRelationWhere() {
        return relationWhere;
    }

    public void setRelationWhere(String relationWhere) {
        this.relationWhere = relationWhere;
    }

    public boolean isFollowDelete() {
        return followDelete;
    }

    public void setFollowDelete(boolean followDelete) {
        this.followDelete = followDelete;
    }

    public boolean isFollowInsert() {
        return followInsert;
    }

    public void setFollowInsert(boolean followInsert) {
        this.followInsert = followInsert;
    }

    public boolean isFollowUpdate() {
        return followUpdate;
    }

    public void setFollowUpdate(boolean followUpdate) {
        this.followUpdate = followUpdate;
    }

    public String getThisColumn() {
        return thisColumn;
    }

    public void setThisColumn(String thisColumn) {
        this.thisColumn = thisColumn;
    }

    public DictObjTable addCondition(DictCondition condition) {
        checkNotNull(condition);
        this.conditions.add(condition);
        return this;
    }

    public void removeCondition(DictCondition condition) {
        if (!this.conditions.contains(condition)) {
            throw new IllegalArgumentException("relation not contain this condition");
        }
        this.conditions.remove(condition);
    }

    public List<DictCondition> getAllCondition() {
        List<DictCondition> tmp = new ArrayList<>();
        tmp.addAll(this.conditions);
        return tmp;
    }

    public static DictObjTable parse(JSONObject schema) {
        DictObjTable tb = new DictObjTable();
        tb.tableName = schema.getString("tableName");
        tb.tableLabel = schema.getString("tableLabel");
        tb.tableAlias = schema.getString("tableAlias");
        tb.relationTable = schema.getString("relationTable");
        tb.relationColumn = schema.getString("relationColumn");
        tb.thisColumn = schema.getString("thisColumn");
        tb.relationWhere = schema.getString("relationWhere");
        tb.followUpdate = schema.getBoolean("followUpdate");
        tb.followDelete = schema.getBoolean("followDelete");
        tb.followInsert = schema.getBoolean("followInsert");
        tb.followUpdate = schema.getBoolean("followUpdate");

        String strCondition = schema.getString("relationCondition");
        if (!StringUtils.isEmpty(strCondition)) {
            if(strCondition.startsWith("[") && strCondition.endsWith("]")) {
                JSONArray jCondition = JSONArray.parseArray(strCondition);
                if (jCondition != null) {
                    for (int i = 0; i < jCondition.size(); i++) {
                        tb.conditions.add(DictCondition.parse(jCondition.getJSONObject(i)));
                    }
                }
            } else {
                if (StringUtils.isEmpty(tb.relationWhere)) {
                    tb.relationWhere = strCondition;
                }
            }
        }

        tb.relationType = RelationType.valueOf(schema.getString("relationType"));

        return tb;
    }

    public JSONObject toJson() {
        JSONObject jsonTable = new JSONObject();
        jsonTable.put("tableName", this.tableName);
        jsonTable.put("tableAlias", this.tableAlias);
        jsonTable.put("tableLabel", this.tableLabel);
        jsonTable.put("relationTable", this.relationTable);
        jsonTable.put("relationColumn", this.relationColumn);
        jsonTable.put("thisColumn", this.thisColumn);
        jsonTable.put("relationType", this.relationType);
        jsonTable.put("relationWhere", this.relationWhere);
        jsonTable.put("followDelete", this.followDelete);
        jsonTable.put("followInsert", this.followInsert);
        jsonTable.put("followUpdate", this.followUpdate);
        JSONArray jsonCondition = new JSONArray();
        for(DictCondition condition : this.conditions) {
            jsonCondition.add(condition.toJson());
        }
        jsonTable.put("relationCondition", jsonCondition.toJSONString());
        return jsonTable;
    }
}
