package top.daxian1218.dust.db.dict;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.db.DictGlobalConfig;
import top.daxian1218.dust.db.DustDbException;
import top.daxian1218.dust.db.base.IDataAdapter;

import java.util.Optional;

/**
 * @author hst on 2020-8-1.
 */
public class SqlBuilderFactory {
    public static SqlBuilder getBuilder() {
        return getBuilder(StringUtils.EMPTY);
    }

    public static SqlBuilder getBuilder(String dbType) {
        if (StringUtils.isEmpty(dbType)) {
            IDataAdapter defaultAdapter = DictGlobalConfig.Instance.getDataAdapter();
            if (defaultAdapter == null) {
                throw new DustDbException("未找到数据适配器");
            }
            dbType = defaultAdapter.getDbType();
        }

        if (DictGlobalConfig.Instance.getSqlBuilders().size() > 0) {
            String finalDbType = dbType;
            Optional<SqlBuilder> opBuilder = DictGlobalConfig.Instance.getSqlBuilders().stream().filter(sqlBuilder -> sqlBuilder.isSupport(finalDbType))
            .findFirst();
            if (opBuilder.isPresent()) {
                return opBuilder.get();
            }
        }

        throw new DustDictException(dbType + "对应的Sql生成器未找到");
    }

    public static SqlBuilder getBuilder(IDataAdapter adapter) {

        String dbType = adapter == null ? StringUtils.EMPTY : adapter.getDbType();
        return getBuilder(dbType);
    }

}
