package top.daxian1218.dust.db.dict;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.id.IIdWorker;
import top.daxian1218.dust.db.DictGlobalConfig;

import java.util.Arrays;
import java.util.Date;

/**
 * @author hst
 */
public class VariableHandler implements IVariableHandler {

    private final String[] VARS = new String[]{"@NOW", "@ID", "@UPDATE"};

    @Override
    public Object getDefaultValue(DictObjColumn dc, DictObjRow row) {
        switch (dc.getDefaultValue())
        {
            case "@NOW":
            case "@UPDATE":
                return new Date();
            case "@ID":
                return DictGlobalConfig.Instance.getIdWorker().nextId();
            default:
                return dc.getDefaultValue();
        }
    }

    @Override
    public Object getUpdateValue(DictObjColumn dc, DictObjRow row) {
        switch (dc.getDefaultValue())
        {
            case "@UPDATE":
                return new Date();
            default:
                return row.get(dc.getColumnName()).getValue();
        }
    }

    @Override
    public Object getVariable(String name) {
        switch (name)
        {
            case "@ID":
                return DictGlobalConfig.Instance.getIdWorker().nextId();
            case "@NOW":
            case "@UPDATE":
                return new Date();
            default:
                return null;
        }
    }

    @Override
    public boolean isHandled(String name) {
        return Arrays.stream(VARS).anyMatch(str -> StringUtils.equals(name, str));
    }
}
