package top.daxian1218.dust.db.dict.condition.expressions;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.db.dict.DustDictException;
import top.daxian1218.dust.db.dict.condition.DictExpression;

/**
 * @author hst
 */
public class ColumnExpression extends DictExpression {
    private String tableName;
    private String columnName;

    public ColumnExpression(String columnName, String tableName) {
        Preconditions.checkNotEmpty(columnName);
        this.tableName = tableName;
        this.columnName = columnName;
    }

    public ColumnExpression(String columnName) {
        this(columnName, null);
    }

    public ColumnExpression() {
    }

    public String getTableName() {
        return tableName;
    }

    public ColumnExpression setTableName(String tableName) {
        this.tableName = tableName;
        return this;
    }

    public String getColumnName() {
        return columnName;
    }

    public ColumnExpression setColumnName(String columnName) {
        this.columnName = columnName;
        return this;
    }

    @Override
    public DictExpression parse(JSONObject jsonObject) {
        Preconditions.checkNotNull(jsonObject);

        if (jsonObject.containsKey("type")) {
            String strType = jsonObject.getString("type");
            if (!StringUtils.equals(strType, ColumnExpression.class.getSimpleName())) {
                throw new DustDictException("Node type of json not compatible ");
            }
        }

        tableName = jsonObject.getString("tableName");
        columnName = jsonObject.getString("columnName");
        return this;
    }

    @Override
    public JSONObject toJson() {
        JSONObject json = new JSONObject();
        json.put("tableName", tableName);
        json.put("columnName", columnName);
        json.put("type", ColumnExpression.class.getSimpleName());
        return json;
    }
}
