package top.daxian1218.dust.db.dict.condition.expressions;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.db.dict.DustDictException;
import top.daxian1218.dust.db.dict.condition.DictCondition;
import top.daxian1218.dust.db.dict.condition.DictExpression;
import top.daxian1218.dust.db.sql.HSqlCommand;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author hst
 */
public class SubTableExpression extends DictExpression {
    private String tableName;
    private String columnName;
    private String alias;
    private List<DictCondition> subConditions = new ArrayList<>();

    public SubTableExpression() {

    }

    public SubTableExpression(HSqlCommand cmd) {

    }

    public SubTableExpression(String tableName, String columnName, DictCondition[] conditions, String alias){
        Preconditions.checkNotEmpty(tableName);

        this.tableName = tableName;
        this.columnName = columnName;
        this.alias = alias;
        if (conditions != null && conditions.length > 0) {
            subConditions.addAll(Arrays.asList(conditions));
        }
    }

    public String getTableName() {
        return tableName;
    }

    public void setTableName(String tableName) {
        this.tableName = tableName;
    }

    public String getColumnName() {
        return columnName;
    }

    public void setColumnName(String columnName) {
        this.columnName = columnName;
    }

    public String getAlias() {
        return alias;
    }

    public void setAlias(String alias) {
        this.alias = alias;
    }

    public List<DictCondition> getSubConditions() {
        return subConditions;
    }

    @Override
    public DictExpression parse(JSONObject jsonObject) {
        Preconditions.checkNotNull(jsonObject);

        if(jsonObject.containsKey("type")) {
            String strType = jsonObject.getString("type");
            if(!StringUtils.equals(strType, SubTableExpression.class.getSimpleName())) {
                throw new DustDictException("类型不符");
            }
        }

        tableName = jsonObject.getString("tableName");
        columnName = jsonObject.getString("columnName");
        alias = jsonObject.getString("alias");

        JSONArray jsonArray = jsonObject.getJSONArray("subConditions");
        if (jsonArray == null) {
            return this;
        }

        for (int i = 0; i < jsonArray.size(); i++) {
            subConditions.add(DictCondition.parse(jsonArray.getJSONObject(i)));
        }

        return this;
    }

    @Override
    public JSONObject toJson() {
        JSONObject json = new JSONObject();
        json.put("type", SubTableExpression.class.getSimpleName());
        json.put("tableName", tableName);
        json.put("columnName", columnName);
        json.put("alias", alias);
        JSONArray subConditionArray = new JSONArray();
        for (DictCondition cdt : subConditions)
        {
            subConditionArray.add(cdt.toJson());
        }
        json.put("subConditions", subConditionArray);

        return json;

    }
}
