package top.daxian1218.dust.db.dict.condition.expressions;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.db.dict.DataType;
import top.daxian1218.dust.db.dict.DustDictException;
import top.daxian1218.dust.db.dict.condition.DictExpression;
import top.daxian1218.dust.db.dict.utils.DataUtil;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author hst
 */
public class ValueListExpression extends DictExpression {

    private List<Object> values = new ArrayList<>();
    private DataType dataType = DataType.STRING;

    public ValueListExpression() {

    }

    public ValueListExpression(Iterable values) {
        for(Object o : values) {
            this.values.add(o);
        }
    }

    public DataType getDataType() {
        return dataType;
    }

    public ValueListExpression setDataType(DataType dataType) {
        this.dataType = dataType;
        return this;
    }

    public Object[] getValues() {
        return values.stream().map(value -> DataUtil.toTypeValue(dataType, value)).toArray();
    }

    @Override
    public DictExpression parse(JSONObject jsonObject) {
        Preconditions.checkNotNull(jsonObject);

        if (jsonObject.containsKey("type")) {
            String strType = jsonObject.getString("type");
            if (!StringUtils.equals(strType, ValueListExpression.class.getSimpleName())) {
                throw new DustDictException("类型不符合");
            }
        }

        if (jsonObject.containsKey("values")) {
            JSONArray arr = jsonObject.getJSONArray("values");
            if (arr != null) {
                arr.forEach(o -> values.add(o));
            }
        }
        return this;
    }

    @Override
    public JSONObject toJson() {
        JSONObject json = new JSONObject();
        json.put("type", ValueListExpression.class.getSimpleName());

        JSONArray arr = new JSONArray();
        arr.addAll(Arrays.asList(getValues()));
        json.put("values", arr);
        return json;
    }
}
