package top.daxian1218.dust.db.dict.express;

import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.DictObjRow;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Function;

import static top.daxian1218.dust.commons.generic.Preconditions.checkNotNull;


/**
 * @author hst on 2020-8-1.
 */
public class Expression {
    private final DictObj dictObj;
    private final String expr;
    private List<ExprItem> exprItems = new ArrayList<>();
    private ExprItem curItem;
    private boolean require = true;
    private String columnName;
    private Lexer lexer;

    public Expression(DictObj obj, String expr) {
        checkNotNull(obj);
        this.dictObj = obj;
        this.expr = expr;
    }

    private void parseExpr() {
        lexer = new Lexer(this.expr);

        for (; ; ) {
            lexer.nextToken();
            if (lexer.getToken() == TokenType.EOF) {
                break;
            }

            if (lexer.getToken() == TokenType.AND
                    || lexer.getToken() == TokenType.OR
                    ) {
                if (exprItems.size() == 0) {
                    throw new ExpressException("syntax error");
                }

                require = lexer.getToken() == TokenType.AND;

            }


            if (lexer.getToken() == TokenType.IDENTIFIER) {
                columnName = lexer.stringVal;
            } else {
                throw new ExpressException("syntax error");
            }

            lexer.nextToken();
            if (lexer.getToken() == TokenType.EOF) {
                throw new ExpressException("syntax error");
            }

            switch (lexer.getToken()) {
                case GT:
                    curItem = new ExprItemGT();
                    exprItems.add(curItem);
                    break;
                case EQ:
                    curItem = new ExprItemEQ();
                    exprItems.add(curItem);
                    break;
                case BANGEQ:
                    curItem = new ExprItemBANGEQ();
                    exprItems.add(curItem);
                    break;
                case GTEQ:
                case LTEQ:
                    curItem = new ExprItemGTEQ();
                    exprItems.add(curItem);
                    break;
                default:
                    throw new ExpressException("operation not supported");
            }

            curItem.setRequire(require);
            curItem.setLeft(columnName);

            lexer.nextToken();
            if (lexer.isEOF()) {
                throw new ExpressException("syntax error");
            }

            switch (lexer.getToken()) {
                case NULL:
                    curItem.setRight(null);
                    break;
                case IDENTIFIER:
                case LITERAL_CHARS:
                    curItem.setRight(lexer.stringVal);
                    break;
                case LITERAL_INT:
                case LITERAL_FLOAT:
                    curItem.setRight(lexer.numberString());
                    break;
                default:
                    throw new ExpressException("value type not support");

            }

        }

    }

    public List<DictObjRow> execute() {
        parseExpr();
        List<DictObjRow> tempRows = new ArrayList<>();
        for (DictObjRow r : dictObj.getRows()) {
            if (check(r)) {
                tempRows.add(r);
            }
        }

        return tempRows;
    }

    public void execute(Function<DictObjRow, Boolean> iterator) {
        parseExpr();
        for (DictObjRow r : dictObj.getRows()) {
            if (check(r)) {
                if (!iterator.apply(r)) {
                    break;
                }
            }
        }
    }

    private boolean check(DictObjRow r) {
        boolean ret = true;
        for (ExprItem item : exprItems) {
            if (!item.isRequire() && item.check(r)) {
                return true;
            }

            ret = ret && item.check(r);
            if (!ret) {
                return false;
            }
        }

        return ret;
    }

}
