package top.daxian1218.dust.db.sql.command;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.db.DustDbException;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.db.sql.SqlCommandType;

/**
 * @author hst
 */
public class HSelectSqlCommand extends HSqlCommand {
    public HSelectSqlCommand(String tableName) {
        resetSql();
        appendSql(tableName);
        setCommandType(SqlCommandType.QUERY);
    }


    @Override
    protected String combineSql() {
        if (StringUtils.isEmpty(getCommandText())) {
            throw new DustDbException("table name is empty");
        }

        StringBuilder desSb = new StringBuilder("SELECT ");

        fields.forEach(col ->
        {
            if (StringUtils.isEmpty(col.getName())) {
                return;
            }

            if (desSb.length() > 7) {
                desSb.append(",");
            }

            desSb.append(col.getName());
        });

        desSb.append(" FROM ");
        desSb.append(getCommandText());

        if (hasWhere()) {
            desSb.append(" WHERE ");
            desSb.append(getWhere());
        }

        if (!isIgnoreOrder()) {
            String orderStr = getOrder();
            if (!StringUtils.isEmpty(orderStr)) {
                desSb.append(" ORDER BY ");
                desSb.append(orderStr);
            }
        }

        return desSb.toString();
    }
}
