package top.daxian1218.dust.db.sql.command;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.data.HDataColumn;
import top.daxian1218.dust.db.DustDbException;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.db.sql.SqlCommandType;

/**
 * @author hst
 */
public class HUpdateSqlCommand extends HSqlCommand {
    private boolean allowUpdatePrimaryKey = true;
    public HUpdateSqlCommand(String table) {
        resetSql();
        appendSql(table);
        this.setCommandType(SqlCommandType.EXEC);
    }

    @Override
    protected String combineSql() {
        if (StringUtils.isEmpty(getCommandText())) {
            throw new DustDbException("table name is empty");
        }

        StringBuilder sb = new StringBuilder("UPDATE " + getCommandText() + " SET ");
        Boolean needComma = false;
        setWhere(StringUtils.EMPTY);
        for(HDataColumn col : fields) {
            if (StringUtils.isEmpty(col.getName()) || col.getName().startsWith(PrefixOld)) {
                continue;
            }

            if (col.isPrimaryKey()) {
                appendWhere(col.getName() + "=:" + PrefixOld + col.getName());
                if (!allowUpdatePrimaryKey) {
                    continue;
                }
            }

            if (needComma) {
                sb.append(",");
            } else {
                needComma = true;
            }

            // 如果列信息定义了配置的值，则不需要使用动态值
            if (col.getProperties().contains(DbValueKey)) {
                sb.append(col.getName() + "=" + col.getProperties().get(DbValueKey));
            } else {
                sb.append(col.getName() + "=:" + col.getName());
            }
        }

        if (!StringUtils.isEmpty(getWhere())) {
            sb.append(" WHERE ");
            sb.append(getWhere());
        } else {
            throw new DustDbException("更新操作必须指定主键");
        }
        return sb.toString();
    }

    public boolean isAllowUpdatePrimaryKey() {
        return allowUpdatePrimaryKey;
    }

    public void setAllowUpdatePrimaryKey(boolean allowUpdatePrimaryKey) {
        this.allowUpdatePrimaryKey = allowUpdatePrimaryKey;
    }
}
