package top.daxian1218.dust.db.sql.pool;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.db.DbConstants;
import top.daxian1218.dust.db.sql.IDataSource;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.logging.Logger;

/**
 * 数据源缓存类，简单的缓存处理，用于缓存当前运行时使用过的数据源，避免类创建的开销
 *
 * @author hst
 */
public class DataSourcePool {
    static Logger logger = Logger.getLogger("DataSourcePool");

    public static final DataSourcePool Default = new DataSourcePool();

    private final Map<String, IDataSource> dataSourceMap = new HashMap<>();
    private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock();


    private DataSourcePool() {
    }

    /**
     * 通过参数key获取对应的数据源（连接池）
     * @param key 按照数据源名称检索存在的数据源
     * @return
     */
    public IDataSource get(String key) {
        if (StringUtils.isEmpty(key)) {
            key = DbConstants.DEFAULT_DB_NAME;
        }

        ReentrantReadWriteLock.ReadLock readLock = lock.readLock();
        readLock.lock();
        try {
            return dataSourceMap.get(key);
        } finally {
            readLock.unlock();
        }
    }

    public DataSourcePool set(String key, IDataSource ds) {
        ReentrantReadWriteLock.WriteLock writeLock = lock.writeLock();
        writeLock.lock();
        try {
            if (StringUtils.isEmpty(key)) {
                key = DbConstants.DEFAULT_DB_NAME;
            }

            if (ds == null) {
                dataSourceMap.remove(key);
            } else {
                dataSourceMap.put(key, ds);
            }
        } finally {
            writeLock.unlock();
        }
        return this;
    }
}
