package top.daxian1218.dust.db.support;

import top.daxian1218.dust.db.DustDbException;
import top.daxian1218.dust.db.base.IDbOperations;
import top.daxian1218.dust.db.support.clickhouse.ClickHouseDbOperations;
import top.daxian1218.dust.db.support.highgo.HighgoDbOperations;
import top.daxian1218.dust.db.support.mssql.SqlServerDbOperations;
import top.daxian1218.dust.db.support.mysql.MySqlDbOperations;
import top.daxian1218.dust.db.support.oracle.OracleDbOperations;
import top.daxian1218.dust.db.support.postgresql.PostgreSqlDbOperations;

import java.util.ArrayList;
import java.util.List;

/**
 * 数据源工厂类
 *
 * @author hst
 */
public class DataBaseFactory {
    private static final List<IDbOperations> dbRegistry = new ArrayList<>();
    static {
        dbRegistry.add(new MySqlDbOperations());
        dbRegistry.add(new OracleDbOperations());
        dbRegistry.add(new SqlServerDbOperations());
        dbRegistry.add(new PostgreSqlDbOperations());
        dbRegistry.add(new ClickHouseDbOperations());
        dbRegistry.add(new HighgoDbOperations());
    }

    public static void register(IDbOperations dataBase) {
        if (!dbRegistry.contains(dataBase)) {
            dbRegistry.add(dataBase);
        }
    }

    public static IDbOperations create(String dbType) {
        if (dbType == null) {
            return new MySqlDbOperations();
        }

        for (IDbOperations db : dbRegistry) {
            if (db.isSupport(dbType)) {
                return db;
            }
        }
        throw new DustDbException("DbOperation not found for dbType:" + dbType);
    }
}
