package top.daxian1218.dust.db.support.highgo;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.DustDbException;
import top.daxian1218.dust.db.base.DbTypes;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.dict.DictObj;
import top.daxian1218.dust.db.dict.SqlBuilder;
import top.daxian1218.dust.db.sql.HSqlCommand;

public class HighgoBuilder extends SqlBuilder {

    @Override
    public long getLastId(IDataAdapter adapter, DictObj destObj, String incrementColumn) throws DustDbException {

        HSqlCommand cmd = new HSqlCommand("SELECT currval(pg_get_serial_sequence('");
        cmd.appendSql(destObj.getTableName());
        cmd.appendSql("','");
        cmd.appendSql(incrementColumn);
        cmd.appendSql("'))");

        HDataTable dt = adapter.query(cmd);
        if (dt.getRows().size() == 1) {
            return TypeUtil.toInteger(dt.getRows().get(0).get("ID"));
        }
        return 0;
    }

    @Override
    public boolean isSupport(String dbType) {
        return StringUtils.equals(dbType, DbTypes.Highgo);
    }

    @Override
    protected String getDbValue(Object o) {
        if (o == null) {
            return "NULL";
        }

        switch (o.getClass().getSimpleName()) {
            case "Integer":
            case "int":
            case "Long":
            case "long":
            case "Short":
            case "short":
            case "Double":
            case "double":
            case "BigDecimal":
            case "Float":
            case "float":
                return TypeUtil.toString(o);
            case "String":
                return "'" + o + "'";
            case "Boolean":
            case "boolean":
                return TypeUtil.toBoolean(o) ? "1" : "0";
            case "Date":
                return "str_to_date('" + TypeUtil.toString(o, "yyyy-MM-dd HH:mm:ss") + "','%Y-%m-%d %Y-%m-%d %r')";
        }

        return TypeUtil.toString(o);
    }
}
