package top.daxian1218.dust.db.support.oracle;

import oracle.jdbc.OracleTypes;
import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.db.DustDbException;
import top.daxian1218.dust.db.base.DbTypes;
import top.daxian1218.dust.db.sql.SqlDbOperations;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.db.sql.ProgramParameter;
import top.daxian1218.dust.db.support.oracle.exception.OracleExceptionFactory;

import java.sql.SQLException;
import java.util.List;

/**
 * 支持Oracle的DataBase实现
 * @author hst
 */
public class OracleDbOperations extends SqlDbOperations {

    private OracleExceptionFactory factory = new OracleExceptionFactory();

    @Override
    protected String getSql(HSqlCommand cmd, boolean isQuery) {
        cmd.setIgnoreOrder(false);
        String sql = cmd.getDbExecuteSql((v, i) -> "?");

        if (isQuery && cmd.getPageSize() > 0) {
            return "SELECT * FROM (SELECT row_.*, rownum row_num_ FROM ("+ sql +
                    ") row_ WHERE rownum <= ?) row__ WHERE row__.row_num_ >= ?";
        }

        return sql;
    }

    @Override
    protected DustDbException formatException(SQLException ex, HSqlCommand cmd) {
        if (ex.getMessage() != null) {
            return factory.getHandler(ex.getMessage()).getException(ex, cmd);
        }

        return DustDbException.fromDbException(ex, cmd);
    }

    @Override
    public void setPageParameter(List<Object> parameters, HSqlCommand cmd) {
        parameters.add(cmd.getEndIndex() + 1);
        parameters.add(cmd.getBeginIndex() + 1);
    }

    @Override
    protected int getSqlType(ProgramParameter p) {
        switch (p.getParamDataType()) {
            case NUMBER:
            case TIMESTAMP:
                return OracleTypes.NUMBER;
            case DATETIME:
                return OracleTypes.DATE;
            case CLOB:
                return OracleTypes.CLOB;
            case CURSOR:
                return OracleTypes.CURSOR;
            case BLOB:
                return OracleTypes.BLOB;
            default:
                return OracleTypes.VARCHAR;
        }
    }

    @Override
    public boolean isSupport(String dbType) {
        return StringUtils.equals(dbType, DbTypes.Oracle);
    }
}
