package top.daxian1218.dust.db.support.postgresql;

import org.apache.commons.lang3.StringUtils;
import top.daxian1218.dust.db.base.DbTypes;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.db.sql.SqlDbOperations;

import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Types;
import java.util.List;

public class PostgreSqlDbOperations  extends SqlDbOperations {
    @Override
    protected String getSql(HSqlCommand cmd, boolean isQuery) {
        String sql = cmd.getDbExecuteSql((v, i) -> "?");
        if (isQuery && cmd.getPageSize() > 0) {
            return sql + " LIMIT ? OFFSET ?";
        }

        return sql;
    }

    @Override
    public void setPageParameter(List<Object> parameters, HSqlCommand cmd) {
        parameters.add(cmd.getPageSize());
        parameters.add(cmd.getBeginIndex());
    }

    @Override
    public boolean isSupport(String dbType) {
        return StringUtils.equals(dbType, DbTypes.PG);
    }

    @Override
    public void fillStatement(PreparedStatement stmt, List<Object> params) {
        if (params == null || stmt == null) {
            return;
        }

        try {
            for (int len = params.size(), i = 0; i < len; i++) {
                Object p = params.get(i);
                int index = i + 1;
                Object dbValue = getDbValue(p, stmt.getConnection());
                if (dbValue != null) {
                    stmt.setObject(index, dbValue);
                } else {
                    stmt.setNull(index, Types.NULL);
                }
            }

        } catch (SQLException ex) {
            throw formatException(ex, null);
        }
    }
}
