package top.daxian1218.dust.db.tenant;

import top.daxian1218.dust.commons.data.HDataRow;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.commons.util.ClassUtil;
import top.daxian1218.dust.db.DbAdapterManager;
import top.daxian1218.dust.db.DustDbException;
import top.daxian1218.dust.db.base.*;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.db.tenant.pojo.TenantApp;
import org.apache.commons.lang3.StringUtils;

/**
 * 配置管理器
 * 根据不同的配置数据库进行重写
 * @author hst
 */
public class AppManager {
    private String dbName;
    private DbAdapterManager dbAdapterManager;
    private static SeparatedCache separatedCache = new SeparatedCache();

    public AppManager(String dbName, DbAdapterManager dbAdapterManager) {
        this.dbAdapterManager = dbAdapterManager;
        this.dbName = dbName;
    }

    /**
     * Application启动后，加载租户信息以及租户的数据库
     */
    public synchronized void init() throws DustDbException {
        if (StringUtils.isEmpty(dbName)) {
            throw new DustDbException("租户信息数据源未指定");
        }

        IDataAdapter adapter = dbAdapterManager.getAdapter(this.dbName, true);

        try {
            //清空缓存
            separatedCache.clear();

            HSqlCommand appCmd = new HSqlCommand(TenantConsts.SQL_APP_DB);
            HDataTable appConfigs = adapter.query(appCmd);
            if (appConfigs.size() > 0) {
                for (int i = 0; i < appConfigs.size(); i++) {
                    HDataRow row = appConfigs.getRows().get(i);
                    TenantApp tenantApp = ClassUtil.mapToObject(row.toMap(), TenantApp.class);
                    if (tenantApp != null) {
                        separatedCache.setApp(tenantApp);
                    }
                }
            }

        } finally {
            adapter.closeQuiet();
        }
    }

    /**
     * 获取App使用的数据库配置
     *
     * @param tenantId
     * @param appId
     * @return
     */
    public TenantApp getAppConfig(String tenantId, String appId) throws DustDbException {
        TenantApp app = separatedCache.getApp(tenantId, appId);
        if (app == null) {
            IDataAdapter sqlDataAdapter = dbAdapterManager.getAdapter(this.dbName);
            HSqlCommand dbCmd = new HSqlCommand(TenantConsts.SQL_FIND_APP_DB);
            dbCmd.setParameter("tenantId", tenantId);
            dbCmd.setParameter("appId", appId);
            HDataTable data = sqlDataAdapter.query(dbCmd);
            sqlDataAdapter.closeQuiet();
            if (data.size() > 0) {
                TenantApp tenantApp = ClassUtil.mapToObject(data.getRows().get(0).toMap(), TenantApp.class);
                if (tenantApp != null) {
                    saveAppConfig(tenantApp);
                    return tenantApp;
                }
            }
        }
        return app;
    }


    /**
     * 设置或更新App信息
     * <strong>注意：</strong>尽量不要自行修改配置信息。把配置的更新交给dustdb自己控制
     *
     * @param tenantApp
     */
    public void saveAppConfig(TenantApp tenantApp) {
        separatedCache.setApp(tenantApp);
    }

}
