package top.daxian1218.dust.db.tenant;

import top.daxian1218.dust.db.tenant.pojo.TenantApp;
import org.apache.commons.lang3.StringUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * 配置缓存管理器
 *
 * @author hst
 */
public class SeparatedCache {
    private Map<String, TenantApp> apps = new HashMap<>();

    /**
     * 根据id获取App配置
     *
     * @param id
     * @return
     */
    public TenantApp getApp(String id) {
        return getLocalAppConfig(id);
    }

    /**
     * 根据租户id和Appid获取App配置
     * @param tenantId
     * @param appId
     * @return
     */
    public TenantApp getApp(String tenantId, String appId) {
        return getLocalAppConfig(getAppKey(tenantId, appId));
    }

    /**
     * 增加App
     * @param app
     */
    public void setApp(TenantApp app) {
        setLocalAppConfig(app);
    }

    public void clear() {
        apps.clear();
    }

    private TenantApp getLocalAppConfig(String key) {
        return apps.get(key);
    }

    private void setLocalAppConfig(TenantApp localTenantApp) {
        if (StringUtils.isEmpty(localTenantApp.getId())
                || StringUtils.isEmpty(localTenantApp.getAppId())
                || StringUtils.isEmpty(localTenantApp.getTenantId())) {
            return;
        }

        apps.put(localTenantApp.getId(), localTenantApp);
        apps.put(getAppKey(localTenantApp.getTenantId(), localTenantApp.getAppId()), localTenantApp);
    }

    private String getAppKey(String tenantId, String appId) {
        return tenantId + ";" + appId;
    }

    //TODO 对接各方缓存数据库
}
