package top.daxian1218.dust.db.tenant;

import com.alibaba.druid.filter.AutoLoad;
import top.daxian1218.dust.db.DbAdapterManager;
import top.daxian1218.dust.db.DustDbException;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.tenant.pojo.TenantApp;

import java.util.logging.Logger;

import static top.daxian1218.dust.commons.generic.Preconditions.checkNotNull;

/**
 * 租户数据适配管理器
 *
 * @author hst
 */
public class TenantManager {
    static Logger logger = Logger.getLogger("TenantManager");

    private final AppManager appManager;
    private final DbAdapterManager dbAdapterManager;
    private boolean enable;
    private Thread syncThread;
    private boolean isStop;


    public TenantManager(DbAdapterManager dbAdapterManager, AppManager appManager) {
        checkNotNull(dbAdapterManager);

        this.dbAdapterManager = dbAdapterManager;
        this.appManager = appManager;
    }

    /**
     * 获取商户对应的SqlAdapter，用于访问租户的数据
     *
     * @param tenantId
     * @param appId
     * @return
     */
    public IDataAdapter getAdapter(String tenantId, String appId) throws DustDbException {
        if (!enable) {
            return dbAdapterManager.getAdapter(null);
        }

        TenantApp app = appManager.getAppConfig(tenantId, appId);
        if (app == null) {
            throw new DustDbException(String.format("db(tenantId:%s, appId: %s) not found", tenantId, appId));
        }

        return dbAdapterManager.getAdapter(app.getDbName());
    }

    public boolean isEnable() {
        return enable;
    }

    public void setEnable(boolean enable) {
        this.enable = enable;
    }

    public void startSync() {
        if (!enable) {
            return;
        }

        if (syncThread == null || !syncThread.isAlive()) {
            syncThread = new Thread(() -> {
                while (!isStop) {
                    try {
                        appManager.init();
                    } catch (DustDbException e) {
                        e.printStackTrace();
                    }
                    try {
                        Thread.sleep(5 * 60 * 1000);
                    } catch (InterruptedException e) {
                        logger.info("tenant sync error :" + e.getMessage());
                    }
                }
            });
            syncThread.setDaemon(true);
            syncThread.start();
        }
    }

    public void stopSync() {
        if (syncThread != null) {
            isStop = true;
            syncThread.interrupt();
            try {
                syncThread.join();
            } catch (InterruptedException ex) {
                logger.info("sync thread interrupt fail");
            } finally {
                syncThread = null;
            }
        }
    }

    public IDataAdapter getAdapter(String name) {
        return dbAdapterManager.getAdapter(name);
    }
}
